#include <windows.h>
#include <commctrl.h>
#include "resource.h"

int whichFileExists(char * f1, char * f2);
void winToUnix(char * winPath, char * unixPath);
void unixToWin(char * unixPath, char * winPath);

BOOL CALLBACK AboutDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
switch(Message) {
	case WM_INITDIALOG:
	return TRUE;
	case WM_COMMAND:
	switch(LOWORD(wParam)) {
		case IDOK:
		EndDialog(hwnd, IDOK);
		break;
	}
	break;
	case WM_CLOSE:
	EndDialog(hwnd, 0);
	break;
	default:
	return FALSE;
}
return TRUE;
}

BOOL CALLBACK MainDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
switch(Message) {
	case WM_INITDIALOG:
	SendMessage(hwnd, WM_SETICON, ICON_SMALL, (LPARAM)LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_PROGICON), IMAGE_ICON, 16, 16, 0));
	SendMessage(hwnd, WM_SETICON, ICON_BIG, (LPARAM)LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_PROGICON)));
	SendDlgItemMessage(hwnd, IDE_WINFILE, EM_LIMITTEXT, (WPARAM)MAX_PATH, 0);
	SendDlgItemMessage(hwnd, IDE_UNIXFILE, EM_LIMITTEXT, (WPARAM)MAX_PATH, 0);
	return TRUE;
	case WM_COMMAND:
	{
	char winFile[MAX_PATH] = "";
	char unixFile[MAX_PATH] = "";
	HANDLE temp;
	OPENFILENAME ofn;
	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hwnd;
	ofn.lpstrFilter = "Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0";
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrDefExt = "txt";
	switch(LOWORD(wParam)) {
		case IDB_WINSEL:
		ofn.lpstrFile = winFile;
		SendDlgItemMessage(hwnd, IDE_UNIXFILE, WM_GETTEXT, (WPARAM)MAX_PATH, (LPARAM)unixFile);
		temp = CreateFile(unixFile, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL);
		if (temp == INVALID_HANDLE_VALUE) {
		ofn.Flags = OFN_EXPLORER | OFN_HIDEREADONLY;
		if(GetOpenFileName(&ofn)) SendDlgItemMessage(hwnd, IDE_WINFILE, WM_SETTEXT, 0, (LPARAM)winFile);
		} else {
		ofn.Flags = OFN_EXPLORER | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY;
		if(GetSaveFileName(&ofn)) SendDlgItemMessage(hwnd, IDE_WINFILE, WM_SETTEXT, 0, (LPARAM)winFile);
		}
		CloseHandle(temp);
		break;
		case IDB_UNIXSEL:
		ofn.lpstrFile = unixFile;
		SendDlgItemMessage(hwnd, IDE_WINFILE, WM_GETTEXT, (WPARAM)MAX_PATH, (LPARAM)winFile);
		temp = CreateFile(winFile, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL);
		if (temp == INVALID_HANDLE_VALUE) {
		ofn.Flags = OFN_EXPLORER | OFN_HIDEREADONLY;
		if(GetOpenFileName(&ofn)) SendDlgItemMessage(hwnd, IDE_UNIXFILE, WM_SETTEXT, 0, (LPARAM)unixFile);
		} else {
		ofn.Flags = OFN_EXPLORER | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY;
		if(GetSaveFileName(&ofn)) SendDlgItemMessage(hwnd, IDE_UNIXFILE, WM_SETTEXT, 0, (LPARAM)unixFile);
		}
		CloseHandle(temp);
		break;
		case IDB_CONVERT:
		SendDlgItemMessage(hwnd, IDE_WINFILE, WM_GETTEXT, (WPARAM)MAX_PATH, (LPARAM)winFile);
		SendDlgItemMessage(hwnd, IDE_UNIXFILE, WM_GETTEXT, (WPARAM)MAX_PATH, (LPARAM)unixFile);
		switch(whichFileExists(winFile, unixFile)) {
			case 1:
			winToUnix(winFile, unixFile);
			break;
			case 2:
			unixToWin(unixFile, winFile);
			break;
			default:
			MessageBox(hwnd, "Neither (or both) of the files\nyou specified exists.", "Error", MB_OK | MB_ICONEXCLAMATION);
			break;
		}
		break;
		case IDB_ABOUT:
		if (DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_ABOUT), hwnd, AboutDlgProc) == -1) {
		MessageBox(hwnd, "Dialog failed!", "Error", MB_OK | MB_ICONINFORMATION);
		}
		break;
		case IDB_EXIT:
		EndDialog(hwnd, 0);
		break;
	}
	break;
	}
	case WM_CLOSE:
	EndDialog(hwnd, 0);
	break;
	default:
	return FALSE;
}
return TRUE;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {

InitCommonControls();

if (DialogBox(hInstance, MAKEINTRESOURCE(IDD_MAIN), NULL, MainDlgProc) == -1) {
MessageBox(NULL, "Create Window Failed!", "Error", MB_OK | MB_ICONINFORMATION);
}

return 0;
}

int whichFileExists(char * f1, char * f2) {
int fileNum = 0;
HANDLE file1 = CreateFile(f1, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL);
HANDLE file2 = CreateFile(f2, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL);
if (file1 != INVALID_HANDLE_VALUE) fileNum = 1;
if (file2 != INVALID_HANDLE_VALUE) fileNum = 2;
if (file1 != INVALID_HANDLE_VALUE && file2 != INVALID_HANDLE_VALUE) fileNum = 0;
if (file1 != INVALID_HANDLE_VALUE) CloseHandle(file1);
if (file2 != INVALID_HANDLE_VALUE) CloseHandle(file2);
return fileNum;
}

void winToUnix(char * winPath, char * unixPath) {
HANDLE winFile = CreateFile(winPath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL);
HANDLE unixFile  = CreateFile(unixPath, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, NULL);
unsigned long winLen = GetFileSize(winFile, NULL);
if(winLen == 0xFFFFFFFF) {
MessageBox(NULL, "File Conversion Error:\nFile too large", "Error", MB_OK | MB_ICONEXCLAMATION);
} else {
char * winText = new char[winLen + 1];
DWORD dwRead;
ReadFile(winFile, winText, winLen, &dwRead, NULL);
winText[winLen] = '\0';
unsigned long i;
unsigned long lBks = 0;
for(i=0;i<winLen;i++) {
if (winText[i] == '\r') lBks++;
}
char * unixText = new char[winLen - lBks + 1];
unsigned long curBrk = 0;
for(i=0;i<winLen - lBks;i++) {
if (winText[i + curBrk] == '\r') {
unixText[i] = '\n';
curBrk++;
} else {
unixText[i] = winText[i + curBrk];
}}
unixText[winLen - lBks] = '\0';
WriteFile(unixFile, unixText, winLen - lBks, &dwRead, NULL);
delete [] winText;
delete [] unixText;
}
CloseHandle(winFile);
CloseHandle(unixFile);
}

void unixToWin(char * unixPath, char * winPath) {
HANDLE unixFile  = CreateFile(unixPath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL);
HANDLE winFile = CreateFile(winPath, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, NULL);
unsigned long unixLen = GetFileSize(unixFile, NULL);
if(unixLen == 0xFFFFFFFF) {
MessageBox(NULL, "File Conversion Error:\nFile too large", "Error", MB_OK | MB_ICONEXCLAMATION);
} else {
char * unixText = new char[unixLen + 1];
DWORD dwRead;
ReadFile(unixFile, unixText, unixLen, &dwRead, NULL);
unixText[unixLen] = '\0';
unsigned long i;
unsigned long lBks = 0;
for(i=0;i<unixLen;i++) {
if (unixText[i] == '\n') lBks++;
}
char * winText = new char[unixLen + lBks + 1];
unsigned long curBrk = 0;
for(i=0;i<unixLen;i++) {
if (unixText[i] == '\n') {
winText[i + curBrk] = '\r';
curBrk++;
winText[i + curBrk] = '\n';
} else {
winText[i + curBrk] = unixText[i];
}}
winText[unixLen + lBks] = '\0';
WriteFile(winFile, winText, unixLen + lBks, &dwRead, NULL);
delete [] unixText;
delete [] winText;
}
CloseHandle(unixFile);
CloseHandle(winFile);
}
